/*
 * Copyright 2015 Luca Beltrame <lbeltrame@kde.org>
 *
 * This file is part of Danbooru Client.
 *
 * Danbooru Client is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Danbooru Client is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Danbooru Client. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "test_danbooru_post.h"
#include "../danboorupost.h"
#include "../utils.h"
#include "../servicebase.h"

#include <QFile>
#include <QJsonDocument>
#include <QJsonObject>



void TestDanbooruPost::initTestCase()
{

}

void TestDanbooruPost::testPostLoading()
{
    QFETCH(QString, jsonFile);
    auto fileName = QFINDTESTDATA(jsonFile);

    QFile file;
    file.setFileName(fileName);
    file.open(QIODevice::ReadOnly | QIODevice::Text);

    bool ok;
    auto data = Danbooru::parseDanbooruResult(file.readAll(), &ok).toList().at(0).toMap();

    if (!ok) {
        QFAIL("JSON file failed to load");
    }

    QFETCH(Danbooru::ApiType, apiType);
    QFETCH(QUrl, boardUrl);

    auto post = new Danbooru::DanbooruPost(data, apiType);
    Danbooru::fixPostUrl(post, boardUrl);

    QFETCH(int, postId);
    QFETCH(int, postWidth);
    QFETCH(int, postHeight);
    QFETCH(int, fileSize);
    QFETCH(QSet<QString>, postTags);
    QFETCH(QUrl, fileUrl);
    QFETCH(QUrl, thumbnailUrl);
    QFETCH(QUrl, sampleUrl);
    QFETCH(Danbooru::DanbooruPost::Rating, rating);

    QCOMPARE(post->id(), postId);
    QCOMPARE(post->width(), postWidth);
    QCOMPARE(post->height(), postHeight);
    QCOMPARE(post->size(), fileSize);
    QCOMPARE(post->tags(), postTags);
    QCOMPARE(post->fileUrl(), fileUrl);
    QCOMPARE(post->thumbnailUrl(), thumbnailUrl);
    QCOMPARE(post->sampleUrl(), sampleUrl);
    QCOMPARE(post->rating(), rating);
    QCOMPARE(post->pixmap().isNull(), true);

}

void TestDanbooruPost::testPostLoadingXml()
{
    QFETCH(QString, jsonFile);
    auto fileName = QFINDTESTDATA(jsonFile);

    QFile file;
    file.setFileName(fileName);
    file.open(QIODevice::ReadOnly | QIODevice::Text);

    bool ok;
    QByteArray data = file.readAll();
    QFETCH(Danbooru::ApiType, apiType);
    auto contents = Danbooru::parseDanbooruResult(data, "post", &ok, apiType).at(0).toMap();
    QCOMPARE(Danbooru::parseDanbooruResult(data, "post", &ok, apiType).length(), 1);

    if (!ok) {
        QFAIL("XML file failed to load");
    }

    QFETCH(QUrl, boardUrl);

    auto post = new Danbooru::DanbooruPost(contents, apiType);
    Danbooru::fixPostUrl(post, boardUrl);

    QFETCH(int, postId);
    QFETCH(int, postWidth);
    QFETCH(int, postHeight);
    QFETCH(int, fileSize);
    QFETCH(QSet<QString>, postTags);
    QFETCH(QUrl, fileUrl);
    QFETCH(QUrl, thumbnailUrl);
    QFETCH(QUrl, sampleUrl);
    QFETCH(Danbooru::DanbooruPost::Rating, rating);

    QCOMPARE(post->id(), postId);
    QCOMPARE(post->width(), postWidth);
    QCOMPARE(post->height(), postHeight);
    QCOMPARE(post->size(), fileSize);
    QCOMPARE(post->tags(), postTags);
    QCOMPARE(post->fileUrl(), fileUrl);
    QCOMPARE(post->thumbnailUrl(), thumbnailUrl);
    QCOMPARE(post->sampleUrl(), sampleUrl);
    QCOMPARE(post->rating(), rating);
    QCOMPARE(post->pixmap().isNull(), true);

}

void TestDanbooruPost::testPostLoading_data()
{
    //TODO: Add API type
    QTest::addColumn<QString>("jsonFile");
    QTest::addColumn<Danbooru::ApiType>("apiType");
    QTest::addColumn<int>("postId");
    QTest::addColumn<int>("postWidth");
    QTest::addColumn<int>("postHeight");
    QTest::addColumn<int>("fileSize");
    QTest::addColumn<QSet< QString >>("postTags");
    QTest::addColumn<QUrl>("fileUrl");
    QTest::addColumn<QUrl>("thumbnailUrl");
    QTest::addColumn<QUrl>("sampleUrl");
    QTest::addColumn<Danbooru::DanbooruPost::Rating>("rating");
    QTest::addColumn<QUrl>("boardUrl");

    // Konachan.com / yande.re
    QTest::newRow("konachan") << "data/konachan.json" // jsonFile
                              << Danbooru::ApiType::KonachanDanbooru
                              << 205621     // id
                              << 1280       // width
                              << 720        // height
                              << 885610     // file_size
                              << QSet<QString>({"azuki_(sayori)", "coconut_(sayori)", "game_cg", "nekopara",
                                                "sayori"})     // tags
                              << QUrl("http://konachan.net/image/eb36c568af4f4e4ea2f59eda7d8c802d/Konachan.com%20-%20205621%20azuki_%28sayori%29%20coconut_%28sayori%29%20game_cg%20nekopara%20sayori.png")
                              << QUrl("http://konachan.net/data/preview/eb/36/eb36c568af4f4e4ea2f59eda7d8c802d.jpg")
                              << QUrl("http://konachan.net/jpeg/eb36c568af4f4e4ea2f59eda7d8c802d/Konachan.com%20-%20205621%20azuki_%28sayori%29%20coconut_%28sayori%29%20game_cg%20nekopara%20sayori.jpg")
                              << Danbooru::DanbooruPost::Safe
                              << QUrl("http://konachan.com");
    // Danbooru

    QTest::newRow("danbooru") << "data/donmai.json"
                              << Danbooru::ApiType::OriginalDanbooru
                              << 2106872
                              << 1200
                              << 993
                              << 514370
                              << QSet<QString>({"1girl", "all_fours", "bare_legs",
                                                "barefoot", "brown_eyes", "brown_hair", "chain",
                                                "collar", "hat", "hecatia_lapislazuli", "long_hair",
                                                "looking_at_viewer", "off_shoulder",
                                                "shirt", "skirt", "solo", "touhou",
                                                "uranaishi_(miraura)" })
                              << QUrl("http://danbooru.donmai.us/data/7ee70315eba5ea4fbda7cd6011c8d833.jpg")
                              << QUrl("http://danbooru.donmai.us/data/preview/7ee70315eba5ea4fbda7cd6011c8d833.jpg")
                              << QUrl("http://danbooru.donmai.us/data/sample/sample-7ee70315eba5ea4fbda7cd6011c8d833.jpg")
                              << Danbooru::DanbooruPost::Safe
                              << QUrl("http://danbooru.donmai.us");

    // Gelbooru
}

void TestDanbooruPost::testPostLoadingXml_data()
{
    //TODO: Add API type
    QTest::addColumn<QString>("jsonFile");
    QTest::addColumn<Danbooru::ApiType>("apiType");
    QTest::addColumn<int>("postId");
    QTest::addColumn<int>("postWidth");
    QTest::addColumn<int>("postHeight");
    QTest::addColumn<int>("fileSize");
    QTest::addColumn<QSet< QString >>("postTags");
    QTest::addColumn<QUrl>("fileUrl");
    QTest::addColumn<QUrl>("thumbnailUrl");
    QTest::addColumn<QUrl>("sampleUrl");
    QTest::addColumn<Danbooru::DanbooruPost::Rating>("rating");
    QTest::addColumn<QUrl>("boardUrl");

    // Konachan.com / yande.re
    QTest::newRow("konachan") << "data/konachan.xml" // jsonFile
                              << Danbooru::ApiType::KonachanDanbooru
                              << 205776     // id
                              << 1200       // width
                              << 900        // height
                              << 815082     // file_size
                              << QSet<QString>({"bloomers", "gym_uniform", "lemon_tea", "purple_eyes",
                                                "purple_hair", "short_hair", "socks",
                                                "sport"})     // tags
                              << QUrl("http://konachan.net/image/ef374c49e410b125c5a8d48c692c2a60/Konachan.com%20-%20205776%20bloomers%20gym_uniform%20lemon_tea%20purple_eyes%20purple_hair%20short_hair%20socks%20sport.jpg")
                              << QUrl("http://konachan.net/data/preview/ef/37/ef374c49e410b125c5a8d48c692c2a60.jpg")
                              << QUrl("http://konachan.net/image/ef374c49e410b125c5a8d48c692c2a60/Konachan.com%20-%20205776%20bloomers%20gym_uniform%20lemon_tea%20purple_eyes%20purple_hair%20short_hair%20socks%20sport.jpg")
                              << Danbooru::DanbooruPost::Safe
                              << QUrl("http://konachan.com");
    // Danbooru

    QTest::newRow("danbooru") << "data/donmai.xml"
                              << Danbooru::ApiType::OriginalDanbooru
                              << 2107611
                              << 1000
                              << 1524
                              << 907930
                              << QSet<QString>({"1girl", "bikini", "bismarck_(kantai_collection)", "blonde_hair",
                                                "blue_eyes", "breasts", "hat", "highres", "kantai_collection",
                                                "long_hair", "looking_at_viewer", "oota_yuuichi", "peaked_cap",
                                                "skirt", "solo", "striped", "striped_bikini", "striped_swimsuit",
                                                "swimsuit"})
                              << QUrl("http://danbooru.donmai.us/data/0f9dbcbd1bad31fe266af9878422039f.png")
                              << QUrl("http://danbooru.donmai.us/data/preview/0f9dbcbd1bad31fe266af9878422039f.jpg")
                              << QUrl("http://danbooru.donmai.us/data/sample/sample-0f9dbcbd1bad31fe266af9878422039f.jpg")
                              << Danbooru::DanbooruPost::Safe
                              << QUrl("http://danbooru.donmai.us");

    // Gelbooru

    QTest::newRow("gelbooru") << "data/gelbooru.xml"
                              << Danbooru::ApiType::Gelbooru
                              << 2825488
                              << 2483
                              << 3500
                              << 0
                              << QSet<QString>({"2girls", "absurdres", "ahoge", "artist_request", "bikini",
                                                "blonde_hair", "blush", "braid", "building", "cameltoe",
                                                "character_request", "city", "cloud", "cover",
                                                 "cover_page", "eating", "flat_chest", "food",
                                                 "hair_intakes", "highres", "ice_cream", "ice_cream_cone",
                                                 "leaning_forward", "leg_lift", "licking", "long_hair",
                                                 "looking_at_viewer", "multiple_girls", "navel", "open_mouth",
                                                 "outdoors", "red_eyes", "ribbon", "sandals", "school_swimsuit",
                                                 "side-tie_bikini", "silver_hair", "sky", "skyscraper",
                                                 "smile", "swimsuit", "thigh_gap", "tongue", "twin_braids",
                                                 "twintails", "two_side_up", "very_long_hair"})
                              << QUrl("http://gelbooru.com/images/92/ba/92ba4abfcb084ee49c0d3c1a08bcc3bf.jpg")
                              << QUrl("http://gelbooru.com/thumbnails/92/ba/thumbnail_92ba4abfcb084ee49c0d3c1a08bcc3bf.jpg")
                              << QUrl("http://gelbooru.com/samples/92/ba/sample_92ba4abfcb084ee49c0d3c1a08bcc3bf.jpg")
                              << Danbooru::DanbooruPost::Safe
                              << QUrl("http://gelbooru.com");
}


QTEST_MAIN(TestDanbooruPost)

